/*********************************
--- serce ---
--- DS    ---

ledowe serduszko
*********************************/

//------- includes ---------------
#include <avr/io.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
//#include <avr/pgmspace.h>

//------ constants ---------------
#define MCLK		1200000
#define SET			1
#define ON			1
#define OFF			0
#define OK			0
#define CLEAR		0
#define ACTIVE		1

//defines ports
#define IO_PORT		PORTB
#define MIC_IN		0
#define REG_IN		1
#define SWITCH		2
#define OUT1		3
#define OUT2		4

//program constans
#define MAX_CYCLE		11
#define MAX_CYCLE_DIV	15
#define MAX_SEKWENCE	0xFF
#define MODE_END		8
#define MODE_ADDRESS	15
#define RANDOM_VAL		26
#define DEFAULT_DELAY	25

//------ func declare --------------
//advenced func
void hard_init(void);
void wdt_set(char value);
void effects(void);
void mode_write(char value);
char mode_read(void);


static volatile char switch_pressed = SET;
static volatile char komp_status = CLEAR;
static char mode = 0;

//####### MAIN ##############################
int main(void)
{		 
	hard_init();
	
	//------ main loop ------------------------
	while(1)
	{
		//if  komp or switch then go effects	
		if ((komp_status) || (switch_pressed))	effects();

		switch_pressed = CLEAR;
		komp_status = CLEAR; 		
		
		sleep_mode();
	}
}
//###############################################

//---------------------------------
void effects(void)
{
	char cycle;
	char sekwence = MAX_SEKWENCE;	
	char direction = 0;
	char ext_temp = 0;
	char int_temp = 0;
	
	//create random cycle count (1...11);
	cycle = (TCNT0 % MAX_CYCLE_DIV);
	cycle += 3;

	//clear switch flag
	if (switch_pressed) cycle = 5;
	switch_pressed = CLEAR;

	//restore mode
	mode = mode_read();

	//-----------------------------	
	while (cycle)
	{
		while (sekwence)
		{
			//change mode-----------------------------	
			if (switch_pressed)
			{
				mode++;
				mode_write(mode);
				sekwence = MAX_SEKWENCE;
				cycle = MAX_CYCLE;
				switch_pressed = CLEAR; 
			}

			//some prepare
			if (mode >= MODE_END) mode = 0;

			switch (mode)
			{
			//-----------------------------	
				case 0:
				{
					if (sekwence > 43) sekwence = 43; 
					if (direction) ext_temp--;
					else ext_temp++;
					if (ext_temp > 80) direction++;
					if (ext_temp < 3) direction = 0;
					int_temp -= 6;
					_delay_ms(DEFAULT_DELAY);			
				};	break;
			
				case 1:
				{
					if (sekwence > 47) sekwence = 47; 
					if (direction) ext_temp--;
					else ext_temp++;
					if (ext_temp > 50) direction++;
					if (ext_temp < 3) direction = 0;
					int_temp = 50;
					_delay_ms(DEFAULT_DELAY);			
				};	break;

				case 2:
				{
					if (sekwence > 43) sekwence = 43; 
					ext_temp = 10;
					int_temp -= 6;
					_delay_ms(DEFAULT_DELAY);			
				};	break;
			
				case 3:
				{
					if (sekwence > 43) sekwence = 43; 
					int_temp -= 6;
					ext_temp = int_temp + 70;
					_delay_ms(DEFAULT_DELAY);			
				};	break;

				case 4:
				{
					if (sekwence > 47) sekwence = 47; 
					if (direction) ext_temp--;
					else ext_temp++;
					if (ext_temp > 50)
					{
						direction++;
						ext_temp = 50;
					}
					if (ext_temp < 3) direction = 0;
					
					if (ext_temp > 40)
						int_temp = (ext_temp - 40) * 20;
					else
						int_temp = 0;
					
					_delay_ms(DEFAULT_DELAY);			
				};	break;

				case 5:
				{
					if (sekwence > 80)
					{
	   				 	sekwence = 80;
						if (direction) direction = 0;
						else direction = SET;
					}
						 
					if (sekwence < 15) 
					{
						if (direction) int_temp = (15 - sekwence) * 15;
						else ext_temp = (15 - sekwence) * 15;
					}
					else
					{
						if (int_temp > 15) int_temp -= 15;
						else int_temp = 0;
						if (ext_temp > 15) ext_temp -= 15;
						else ext_temp = 0;
					}
					_delay_ms(DEFAULT_DELAY);			
				};	break;
			
				case 6:
				{
					if (sekwence > 80) sekwence = 80; 
					if (sekwence < 5) ext_temp = (10 - sekwence) * 20;
					if (sekwence < 20) int_temp = (20 - sekwence) * 10;
					else
					{
						if (int_temp > 10) int_temp -= 10;
						else int_temp = 0;
						if (ext_temp > 5) ext_temp -= 5;
						else ext_temp = 0;
					}
					
					_delay_ms(DEFAULT_DELAY);			
				};	break;
			
				case 7:
				{
					if (sekwence > 80) sekwence = 80; 
					if (sekwence < 5) ext_temp = (10 - sekwence) * 20;
					if (sekwence < 20) int_temp = (10 - sekwence) * 10;
					else
					{
						if (int_temp > 10) int_temp -= 10;
						else int_temp = 0;
						if (ext_temp > 5) ext_temp -= 5;
						else ext_temp = 0;
					}
					
					_delay_ms(DEFAULT_DELAY);			
				};	break;
			}	
			OCR0A = ext_temp;
			OCR0B = int_temp;	
			sekwence--;
		}
		sekwence = 0xFF;
		cycle--;
	}

	//off led ports
	while ((ext_temp) || (int_temp))
	{
		if (ext_temp) ext_temp--;
		if (int_temp) int_temp--;
		OCR0A = ext_temp;
		OCR0B = ext_temp;
		_delay_ms(50);			
	} 
	IO_PORT &= ~((1<<OUT1) | (1<<OUT2));
}

//---------------------------------
void hard_init(void)
{		
	//config ports
	IO_PORT = (1<<SWITCH);
	DDRB = (1<<OUT1 | 1<<OUT2);
	DIDR0 = (1<<AIN0D | 1<<AIN1D);
	
	//config COMP
	ACSR = (1<<ACIE);

	//enable sleep
	set_sleep_mode(SLEEP_MODE_IDLE);
	
	//config timer
	TCCR0A = ((1<<WGM00) | (1<<WGM01));	//fast PWM
	TCCR0B = ((1<<CS00) | (1<<CS01));	//presc 64

	//config timer interrupts
	TIMSK0 = ((1<<OCIE0A) | (1<<OCIE0B) | (1<<TOIE0));

	//config pin interrupts
	PCMSK = (1<<PCINT2);
	GIMSK = (1<<PCIE);
	
	//enable all int
	sei();

}

//--------------------------------- 
void mode_write(char value)
{
	while(EECR & (1<<EEPE)); 		//Wait for completion of previous write
	EECR = (0<<EEPM1)|(0>>EEPM0);	//Set Programming mode

	EEARL = MODE_ADDRESS;				//Set up address and data registers
	EEDR = value;

	EECR |= (1<<EEMPE);				//Write logical one to EEMPE
	EECR |= (1<<EEPE);				//Start eeprom write by setting EEPE
}

//--------------------------------- 
char mode_read(void)
{
	while(EECR & (1<<EEPE));		//Wait for completion of previous write

	EEARL = MODE_ADDRESS;				//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE

	return EEDR;					//Return data from data register
}

//---------------------------------
void wdt_set(char value)
{
	wdt_reset();		
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = value;
}

//---------------------------------
SIGNAL(PCINT0_vect)
{
	_delay_ms(1);
	if (!(PINB & (1<<SWITCH))) switch_pressed = SET;
}

//-------------------------------------
SIGNAL(TIM0_COMPA_vect)	
{
	//set leds 1;
	IO_PORT &= ~(1<<OUT1);
}

//-------------------------------------
SIGNAL(TIM0_COMPB_vect)
{
	//set leds 1;
	IO_PORT &= ~(1<<OUT2);
}

//-------------------------------------
SIGNAL(TIM0_OVF_vect)
{
	IO_PORT &= ~((1<<OUT1) | (1<<OUT2));	//off both
	if (OCR0A) IO_PORT |= (1<<OUT1); 		//set if needed
	if (OCR0B) IO_PORT |= (1<<OUT2); 
}


//-------------------------------------
SIGNAL(ANA_COMP_vect)
{
	komp_status = SET;
}


